@@ -1,4 +1,5 @@
 """Logic related to validators applied to models etc. via the `@field_validator` and `@model_validator` decorators."""
+
 from __future__ import annotations as _annotations
 
 from collections import deque
@@ -14,7 +15,9 @@ from typing_extensions import Literal, TypeAlias, is_typeddict
 from ..errors import PydanticUserError
 from ._core_utils import get_type_ref
 from ._internal_dataclass import slots_true
+from ._namespace_utils import GlobalsNamespace, MappingNamespace
 from ._typing_extra import get_function_type_hints
+from ._utils import can_be_positional
 
 if TYPE_CHECKING:
     from ..fields import ComputedFieldInfo
@@ -55,6 +58,9 @@ class FieldValidatorDecoratorInfo:
         fields: A tuple of field names the validator should be called on.
         mode: The proposed validator mode.
         check_fields: Whether to check that the fields actually exist on the model.
+        json_schema_input_type: The input type of the function. This is only used to generate
+            the appropriate JSON Schema (in validation mode) and can only specified
+            when `mode` is either `'before'`, `'plain'` or `'wrap'`.
     """
 
     decorator_repr: ClassVar[str] = '@field_validator'
@@ -62,6 +68,7 @@ class FieldValidatorDecoratorInfo:
     fields: tuple[str, ...]
     mode: FieldValidatorModes
     check_fields: bool | None
+    json_schema_input_type: Any
 
 
 @dataclass(**slots_true)
@@ -126,7 +133,7 @@ class ModelValidatorDecoratorInfo:
     while building the pydantic-core schema.
 
     Attributes:
-        decorator_repr: A class variable representing the decorator string, '@model_serializer'.
+        decorator_repr: A class variable representing the decorator string, '@model_validator'.
         mode: The proposed serializer mode.
     """
 
@@ -177,6 +184,12 @@ class PydanticDescriptorProxy(Generic[ReturnType]):
 
     def _call_wrapped_attr(self, func: Callable[[Any], None], *, name: str) -> PydanticDescriptorProxy[ReturnType]:
         self.wrapped = getattr(self.wrapped, name)(func)
+        if isinstance(self.wrapped, property):
+            # update ComputedFieldInfo.wrapped_property
+            from ..fields import ComputedFieldInfo
+
+            if isinstance(self.decorator_info, ComputedFieldInfo):
+                self.decorator_info.wrapped_property = self.wrapped
         return self
 
     def __get__(self, obj: object | None, obj_type: type[object] | None = None) -> PydanticDescriptorProxy[ReturnType]:
@@ -494,7 +507,7 @@ class DecoratorInfos:
             # so then we don't need to re-process the type, which means we can discard our descriptor wrappers
             # and replace them with the thing they are wrapping (see the other setattr call below)
             # which allows validator class methods to also function as regular class methods
-            setattr(model_dc, '__pydantic_decorators__', res)
+            model_dc.__pydantic_decorators__ = res
             for name, value in to_replace:
                 setattr(model_dc, name, value)
         return res
@@ -514,12 +527,11 @@ def inspect_validator(validator: Callable[..., Any], mode: FieldValidatorModes)
     """
     try:
         sig = signature(validator)
-    except ValueError:
-        # builtins and some C extensions don't have signatures
-        # assume that they don't take an info argument and only take a single argument
-        # e.g. `str.strip` or `datetime.datetime`
+    except (ValueError, TypeError):
+        # `inspect.signature` might not be able to infer a signature, e.g. with C objects.
+        # In this case, we assume no info argument is present:
         return False
-    n_positional = count_positional_params(sig)
+    n_positional = count_positional_required_params(sig)
     if mode == 'wrap':
         if n_positional == 3:
             return True
@@ -538,9 +550,7 @@ def inspect_validator(validator: Callable[..., Any], mode: FieldValidatorModes)
     )
 
 
-def inspect_field_serializer(
-    serializer: Callable[..., Any], mode: Literal['plain', 'wrap'], computed_field: bool = False
-) -> tuple[bool, bool]:
+def inspect_field_serializer(serializer: Callable[..., Any], mode: Literal['plain', 'wrap']) -> tuple[bool, bool]:
     """Look at a field serializer function and determine if it is a field serializer,
     and whether it takes an info argument.
 
@@ -549,18 +559,21 @@ def inspect_field_serializer(
     Args:
         serializer: The serializer function to inspect.
         mode: The serializer mode, either 'plain' or 'wrap'.
-        computed_field: When serializer is applied on computed_field. It doesn't require
-            info signature.
 
     Returns:
         Tuple of (is_field_serializer, info_arg).
     """
-    sig = signature(serializer)
+    try:
+        sig = signature(serializer)
+    except (ValueError, TypeError):
+        # `inspect.signature` might not be able to infer a signature, e.g. with C objects.
+        # In this case, we assume no info argument is present and this is not a method:
+        return (False, False)
 
     first = next(iter(sig.parameters.values()), None)
     is_field_serializer = first is not None and first.name == 'self'
 
-    n_positional = count_positional_params(sig)
+    n_positional = count_positional_required_params(sig)
     if is_field_serializer:
         # -1 to correct for self parameter
         info_arg = _serializer_info_arg(mode, n_positional - 1)
@@ -572,13 +585,8 @@ def inspect_field_serializer(
             f'Unrecognized field_serializer function signature for {serializer} with `mode={mode}`:{sig}',
             code='field-serializer-signature',
         )
-    if info_arg and computed_field:
-        raise PydanticUserError(
-            'field_serializer on computed_field does not use info signature', code='field-serializer-signature'
-        )
 
-    else:
-        return is_field_serializer, info_arg
+    return is_field_serializer, info_arg
 
 
 def inspect_annotated_serializer(serializer: Callable[..., Any], mode: Literal['plain', 'wrap']) -> bool:
@@ -593,8 +601,13 @@ def inspect_annotated_serializer(serializer: Callable[..., Any], mode: Literal['
     Returns:
         info_arg
     """
-    sig = signature(serializer)
-    info_arg = _serializer_info_arg(mode, count_positional_params(sig))
+    try:
+        sig = signature(serializer)
+    except (ValueError, TypeError):
+        # `inspect.signature` might not be able to infer a signature, e.g. with C objects.
+        # In this case, we assume no info argument is present:
+        return False
+    info_arg = _serializer_info_arg(mode, count_positional_required_params(sig))
     if info_arg is None:
         raise PydanticUserError(
             f'Unrecognized field_serializer function signature for {serializer} with `mode={mode}`:{sig}',
@@ -622,7 +635,7 @@ def inspect_model_serializer(serializer: Callable[..., Any], mode: Literal['plai
         )
 
     sig = signature(serializer)
-    info_arg = _serializer_info_arg(mode, count_positional_params(sig))
+    info_arg = _serializer_info_arg(mode, count_positional_required_params(sig))
     if info_arg is None:
         raise PydanticUserError(
             f'Unrecognized model_serializer function signature for {serializer} with `mode={mode}`:{sig}',
@@ -635,18 +648,18 @@ def inspect_model_serializer(serializer: Callable[..., Any], mode: Literal['plai
 def _serializer_info_arg(mode: Literal['plain', 'wrap'], n_positional: int) -> bool | None:
     if mode == 'plain':
         if n_positional == 1:
-            # (__input_value: Any) -> Any
+            # (input_value: Any, /) -> Any
             return False
         elif n_positional == 2:
-            # (__model: Any, __input_value: Any) -> Any
+            # (model: Any, input_value: Any, /) -> Any
             return True
     else:
         assert mode == 'wrap', f"invalid mode: {mode!r}, expected 'plain' or 'wrap'"
         if n_positional == 2:
-            # (__input_value: Any, __serializer: SerializerFunctionWrapHandler) -> Any
+            # (input_value: Any, serializer: SerializerFunctionWrapHandler, /) -> Any
             return False
         elif n_positional == 3:
-            # (__input_value: Any, __serializer: SerializerFunctionWrapHandler, __info: SerializationInfo) -> Any
+            # (input_value: Any, serializer: SerializerFunctionWrapHandler, info: SerializationInfo, /) -> Any
             return True
 
     return None
@@ -707,27 +720,25 @@ def unwrap_wrapped_function(
     unwrap_class_static_method: bool = True,
 ) -> Any:
     """Recursively unwraps a wrapped function until the underlying function is reached.
-    This handles property, functools.partial, functools.partialmethod, staticmethod and classmethod.
+    This handles property, functools.partial, functools.partialmethod, staticmethod, and classmethod.
 
     Args:
         func: The function to unwrap.
-        unwrap_partial: If True (default), unwrap partial and partialmethod decorators, otherwise don't.
-            decorators.
+        unwrap_partial: If True (default), unwrap partial and partialmethod decorators.
         unwrap_class_static_method: If True (default), also unwrap classmethod and staticmethod
             decorators. If False, only unwrap partial and partialmethod decorators.
 
     Returns:
         The underlying function of the wrapped function.
     """
-    all: set[Any] = {property, cached_property}
+    # Define the types we want to check against as a single tuple.
+    unwrap_types = (
+        (property, cached_property)
+        + ((partial, partialmethod) if unwrap_partial else ())
+        + ((staticmethod, classmethod) if unwrap_class_static_method else ())
+    )
 
-    if unwrap_partial:
-        all.update({partial, partialmethod})
-
-    if unwrap_class_static_method:
-        all.update({staticmethod, classmethod})
-
-    while isinstance(func, tuple(all)):
+    while isinstance(func, unwrap_types):
         if unwrap_class_static_method and isinstance(func, (classmethod, staticmethod)):
             func = func.__func__
         elif isinstance(func, (partial, partialmethod)):
@@ -743,7 +754,10 @@ def unwrap_wrapped_function(
 
 
 def get_function_return_type(
-    func: Any, explicit_return_type: Any, types_namespace: dict[str, Any] | None = None
+    func: Any,
+    explicit_return_type: Any,
+    globalns: GlobalsNamespace | None = None,
+    localns: MappingNamespace | None = None,
 ) -> Any:
     """Get the function return type.
 
@@ -753,7 +767,8 @@ def get_function_return_type(
     Args:
         func: The function to get its return type.
         explicit_return_type: The explicit return type.
-        types_namespace: The types namespace, defaults to `None`.
+        globalns: The globals namespace to use during type annotation evaluation.
+        localns: The locals namespace to use during type annotation evaluation.
 
     Returns:
         The function return type.
@@ -761,19 +776,36 @@ def get_function_return_type(
     if explicit_return_type is PydanticUndefined:
         # try to get it from the type annotation
         hints = get_function_type_hints(
-            unwrap_wrapped_function(func), include_keys={'return'}, types_namespace=types_namespace
+            unwrap_wrapped_function(func),
+            include_keys={'return'},
+            globalns=globalns,
+            localns=localns,
         )
         return hints.get('return', PydanticUndefined)
     else:
         return explicit_return_type
 
 
-def count_positional_params(sig: Signature) -> int:
-    return sum(1 for param in sig.parameters.values() if can_be_positional(param))
+def count_positional_required_params(sig: Signature) -> int:
+    """Get the number of positional (required) arguments of a signature.
 
+    This function should only be used to inspect signatures of validation and serialization functions.
+    The first argument (the value being serialized or validated) is counted as a required argument
+    even if a default value exists.
 
-def can_be_positional(param: Parameter) -> bool:
-    return param.kind in (Parameter.POSITIONAL_ONLY, Parameter.POSITIONAL_OR_KEYWORD)
+    Returns:
+        The number of positional arguments of a signature.
+    """
+    parameters = list(sig.parameters.values())
+    return sum(
+        1
+        for param in parameters
+        if can_be_positional(param)
+        # First argument is the value being validated/serialized, and can have a default value
+        # (e.g. `float`, which has signature `(x=0, /)`). We assume other parameters (the info arg
+        # for instance) should be required, and thus without any default value.
+        and (param.default is Parameter.empty or param is parameters[0])
+    )
 
 
 def ensure_property(f: Any) -> Any:
