@@ -1,4 +1,5 @@
 """Pydantic-specific errors."""
+
 from __future__ import annotations as _annotations
 
 import re
@@ -36,6 +37,7 @@ PydanticErrorCodes = Literal[
     'model-field-missing-annotation',
     'config-both',
     'removed-kwargs',
+    'circular-reference-schema',
     'invalid-for-json-schema',
     'json-schema-already-used',
     'base-model-instantiated',
@@ -47,6 +49,7 @@ PydanticErrorCodes = Literal[
     'validator-no-fields',
     'validator-invalid-fields',
     'validator-instance-method',
+    'validator-input-type',
     'root-validator-pre-skip',
     'model-serializer-instance-method',
     'validator-field-config-info',
@@ -55,13 +58,19 @@ PydanticErrorCodes = Literal[
     'field-serializer-signature',
     'model-serializer-signature',
     'multiple-field-serializers',
-    'invalid_annotated_type',
+    'invalid-annotated-type',
     'type-adapter-config-unused',
     'root-model-extra',
     'unevaluable-type-annotation',
     'dataclass-init-false-extra-allow',
     'clashing-init-and-init-var',
     'model-config-invalid-field-name',
+    'with-config-on-model',
+    'dataclass-on-model',
+    'validate-call-type',
+    'unpack-typed-dict',
+    'overlapping-unpack-typed-dict',
+    'invalid-self-type',
 ]
 
 
